// This file was generated by ecm_setup_version(): DO NOT EDIT!

#ifndef KSCREEN_VERSION_H
#define KSCREEN_VERSION_H

#define KSCREEN_VERSION_STRING "5.26.5"
#define KSCREEN_VERSION_MAJOR 5
#define KSCREEN_VERSION_MINOR 26
#define KSCREEN_VERSION_PATCH 5
#define KSCREEN_VERSION ((5<<16)|(26<<8)|(5))

#endif
